﻿import logging
from urllib.parse import urlencode, quote_plus
from urllib.request import urlopen
from xml.dom.minidom import *

# Класс python для отправки СМС и Viber сообщений
# и контролировать состояние ранее отправленных сообщений
# перед вызовом метода sendSMS (отправка сообщений) необходимо присвоить значения
# параметрам username и password
# если параметр isViber принимает значение True, то сообщения отправляются на Viber
class Websms(object):

    def __init__(self):
        self._username = ''
        self._password = ''
        self._fromName = ''
        self._isViber = False 
        self._messId = []
        self._websmsLogger = logging.getLogger('websms')
        # fh = logging.FileHandler('websms-notify.log')
        # self._websmsLogger.addHandler(fh) 

    @property
    def username(self):
        return self._username

    @username.setter
    def username(self, val):
        self._username = val
    
    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, val):
        self._password = val

    @property
    def fromName(self):
        return self._fromName

    @fromName.setter
    def fromName(self, val):
        self._fromName = val
    
    @property
    def messId(self):
        return self._messId

    @property
    def isViber(self):
        return self._isViber

    @isViber.setter
    def isViber(self, val):
        boolVal = True if val else False
        self._isViber = boolVal
    
    # Метод, отправляющий СМС сообщение с текстом msg на один адресат, параметр phone 
    # (или несколько адресатов, перечисленные через запятую)
    def sendSMS(self, msg, phone):
        if self._username == '' or self._password == '':
            self._websmsLogger.error('Login or password is empty')
            return False
        elif msg == '':
            self._websmsLogger.error('You cannot send empty message')
            return False
        elif phone == '':
            self._websmsLogger.error('Non-empty list of recipients requires')
            return False
        else: 
            msg = msg.encode('windows-1251')
            url = 'http://websms.ru/http_in5.asp'
            mode = 'SMS'
            if self._isViber:
                mode = 'VIBER'
            params = {'http_username':self._username, 'http_password':self._password, 'fromPhone':self._fromName, 'Message':msg, 'Phone_list':phone, 'format':'XML', 'mode':mode}
            url = url + '?' + urlencode(params, quote_via=quote_plus)

            self._websmsLogger.debug('Send SMS with: %s' % url)
            try:
                response = urlopen(url)

                if response.code != 200:
                    self._websmsLogger.error('Falied send SMS: status %s - %s' % (response.code, response.read()))
                    return False
                
                xml = response.read()
                dom = parseString(xml)
                httpIn = dom.getElementsByTagName('httpIn')[0]
                error_num = httpIn.getAttributeNode('error_num').nodeValue
                if error_num != '0':
                    self._websmsLogger.error('Falied send SMS: server returned error_num %s' % error_num)
                    return False
                else:
                    sms = dom.getElementsByTagName('sms')
                    for node in sms:
                        msgId = node.getAttributeNode('message_id').nodeValue
                        if msgId is not None:
                            self._messId.append(msgId)
                            
            except Exception as err:
                self._websmsLogger.error('Falied send SMS: %s' % err)
                return False
    
            self._websmsLogger.debug('the message was sent')
            return True

        
    # Метод, запрашивающий состояние ранее отправленного СМС-сообщения
    # по ID сообщения
    # расшифровка возвращаемых значений - http://websms.ru/content/doc/HTTP_HTTPSsendmethod_v1.9.1.pdf стр.13
    # основные значения: 4 - передано оператору, 5 - не принято, 6 - доставлено адресату, 7 - недоставляемо
    def getStatus(self, msgId):
        result_id = 0
        url = 'http://websms.ru/http_out5.asp'
        params = {'http_username':self._username, 'http_password':self._password, 'message_id':msgId, 'format':'XML'}
        url = url + '?' + urlencode(params, quote_via=quote_plus)

        self._websmsLogger.debug('Send Status request for message_id %s' % msgId)
        try:
            response = urlopen(url)

            if response.code != 200:
                self._websmsLogger.error('Falied Status request: status %s - %s' % (response.code, response.read()))
                return 0
                
            xml = response.read()
            dom = parseString(xml)
            sms = dom.getElementsByTagName('sms')
            for node in sms:
                result_id = int(node.getAttributeNode('result_id').nodeValue)
                break
                            
        except Exception as err:
            self._websmsLogger.error('Falied Status request: %s' % err)

        return result_id
    
    # Метод, запрашивающий состояние последнего отправленного СМС-сообщения
    # расшифровка возвращаемых значений - http://websms.ru/content/doc/HTTP_HTTPSsendmethod_v1.9.1.pdf стр.13
    # основные значения: 4 - передано оператору, 5 - не принято, 6 - доставлено адресату, 7 - недоставляемо
    def getLastStatus(self):
        if len(self._messId)>0:
            return self.getStatus(self._messId[len(self._messId)-1])
        else:
            return 0

    # Метод, возвращающий ID последнего отправленного СМС-сообщения
    def getLastMessId(self):
        if len(self._messId)>0:
            return self._messId[len(self._messId)-1]
        else:
            return 0

    # Метод, очищающий список ID отправленных ранее сообщений
    def clearMsg(self):
        self._messId.clear() 
                

#  ПРИМЕР ИСПОЛЬЗОВАНИЯ КЛАССА
w = Websms();
w.username = 'логин'
w.password = 'пароль'
w.isViber = True
w.sendSMS('test python websms class','79212345678')
print('message ID = %s' % w.getLastMessId())
print('Состояние последнего СМС-сообщения: %s' % w.getLastStatus())
